@@ -1,11 +1,15 @@
+from __future__ import annotations
+
 import argparse
+import copy
 import logging
+import multiprocessing
 import sys
 import uuid
 import zipfile
 from dataclasses import dataclass
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Optional, Set, Union
+from typing import Any, Dict, Iterator, List, Optional, Union
 
 import annofabapi
 from annofabapi.dataclass.annotation import AdditionalData, AnnotationDetail
@@ -13,7 +17,7 @@ from annofabapi.models import (
     AdditionalDataDefinitionType,
     AdditionalDataDefinitionV1,
     AnnotationDataHoldingType,
-    AnnotationType,
+    DefaultAnnotationType,
     LabelV1,
     ProjectMemberRole,
     TaskStatus,
@@ -29,7 +33,6 @@ from dataclasses_json import DataClassJsonMixin
 from more_itertools import first_true
 
 import annofabcli
-from annofabcli import AnnofabApiFacade
 from annofabcli.common.cli import (
     COMMAND_LINE_ERROR_STATUS_CODE,
     AbstractCommandLineInterface,
@@ -37,6 +40,7 @@ from annofabcli.common.cli import (
     ArgumentParser,
     build_annofabapi_resource_and_login,
 )
+from annofabcli.common.facade import AnnofabApiFacade
 from annofabcli.common.visualize import AddProps, MessageLocale
 
 logger = logging.getLogger(__name__)
@@ -130,13 +134,13 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
         """
         data_uri = annotation_data["data"]
         # この時点で data_uriは f"./{input_data_id}/{annotation_id}"
-        # paraser.open_data_uriメソッドに渡す値は、先頭のinput_data_idは不要なので、これを取り除く
+        # parser.open_data_uriメソッドに渡す値は、先頭のinput_data_idは不要なので、これを取り除く
         path = Path(data_uri)
         return str(path.relative_to(path.parts[0]))
 
     @classmethod
     def _is_3dpc_segment_label(cls, label_info: Dict[str, Any]) -> bool:
-        if label_info["annotation_type"] != AnnotationType.CUSTOM.value:
+        if label_info["annotation_type"] != DefaultAnnotationType.CUSTOM.value:
             return False
 
         metadata = label_info["metadata"]
@@ -146,12 +150,13 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
 
     @classmethod
     def _get_data_holding_type_from_data(cls, label_info: Dict[str, Any]) -> AnnotationDataHoldingType:
-        annotation_type = AnnotationType(label_info["annotation_type"])
-        if annotation_type in [AnnotationType.SEGMENTATION, AnnotationType.SEGMENTATION_V2]:
+
+        annotation_type = label_info["annotation_type"]
+        if annotation_type in [DefaultAnnotationType.SEGMENTATION.value, DefaultAnnotationType.SEGMENTATION_V2.value]:
             return AnnotationDataHoldingType.OUTER
 
         # TODO: 3dpc editorに依存したコード。annofab側でSimple Annotationのフォーマットが改善されたら、このコードを削除する
-        if annotation_type == AnnotationType.CUSTOM:
+        if annotation_type == DefaultAnnotationType.CUSTOM.value:
             if cls._is_3dpc_segment_label(label_info):
                 return AnnotationDataHoldingType.OUTER
         return AnnotationDataHoldingType.INNER
@@ -215,7 +220,7 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
             if detail.annotation_id is not None:
                 return detail.annotation_id
             else:
-                if arg_label_info["annotation_type"] == AnnotationType.CLASSIFICATION.value:
+                if arg_label_info["annotation_type"] == DefaultAnnotationType.CLASSIFICATION.value:
                     # 全体アノテーションの場合、annotation_idはlabel_idである必要がある
                     return arg_label_info["label_id"]
                 else:
@@ -253,7 +258,6 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
             with parser.open_outer_file(data_uri) as f:
                 s3_path = self.service.wrapper.upload_data_to_s3(self.project_id, f, content_type="image/png")
                 dest_obj.path = s3_path
-                logger.debug(f"{parser.task_id}/{parser.input_data_id}/{data_uri} をS3にアップロードしました。")
 
         return dest_obj
 
@@ -266,7 +270,15 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
         request_details: List[Dict[str, Any]] = []
         now_datetime = str_now()
         for detail in details:
-            request_detail = self._to_annotation_detail_for_request(parser, detail, now_datetime=now_datetime)
+            try:
+                request_detail = self._to_annotation_detail_for_request(parser, detail, now_datetime=now_datetime)
+            except Exception:
+                logger.warning(
+                    f"{parser.task_id}/{parser.input_data_id} :: アノテーションをrequest_bodyに変換するのに失敗しました。 :: "
+                    f"annotation_id={detail.annotation_id}, label={detail.label}",
+                    exc_info=True,
+                )
+                continue
 
             if request_detail is not None:
                 request_details.append(request_detail.to_dict(encode_json=True))
@@ -294,6 +306,10 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
         old_dict_detail = {}
         INDEX_KEY = "_index"
         for index, old_detail in enumerate(old_details):
+            if old_detail["data_holding_type"] == AnnotationDataHoldingType.OUTER.value:
+                # 外部アノテーションを利用する際はurlが不要でpathが必要なので、対応する
+                old_detail.pop("url", None)
+
             # 一時的にインデックスを格納
             old_detail.update({INDEX_KEY: index})
             old_dict_detail[old_detail["annotation_id"]] = old_detail
@@ -301,7 +317,15 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
         new_request_details: List[Dict[str, Any]] = []
         now_datetime = str_now()
         for detail in details:
-            request_detail = self._to_annotation_detail_for_request(parser, detail, now_datetime=now_datetime)
+            try:
+                request_detail = self._to_annotation_detail_for_request(parser, detail, now_datetime=now_datetime)
+            except Exception:
+                logger.warning(
+                    f"{parser.task_id}/{parser.input_data_id} :: アノテーションをrequest_bodyに変換するのに失敗しました。 :: "
+                    f"annotation_id={detail.annotation_id}, label={detail.label}",
+                    exc_info=True,
+                )
+                continue
 
             if request_detail is None:
                 continue
@@ -314,11 +338,13 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
                 # アノテーションの追加
                 new_request_details.append(request_detail.to_dict(encode_json=True))
 
+        new_details = old_details + new_request_details
+
         request_body = {
             "project_id": self.project_id,
             "task_id": parser.task_id,
             "input_data_id": parser.input_data_id,
-            "details": old_details + new_request_details,
+            "details": new_details,
             "updated_datetime": old_annotation["updated_datetime"],
         }
 
@@ -373,14 +399,13 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
                     success_count += 1
             except Exception:  # pylint: disable=broad-except
                 logger.warning(
-                    f"task_id={parser.task_id}, input_data_id={parser.input_data_id} の" f"アノテーションインポートに失敗しました。",
+                    f"task_id={parser.task_id}, input_data_id={parser.input_data_id} の" f"アノテーションのインポートに失敗しました。",
                     exc_info=True,
                 )
 
-        logger.info(f"タスク'{task_parser.task_id}'の入力データ {success_count} 個に対してアノテーションをインポートしました。")
         return success_count
 
-    def execute_task(self, task_parser: SimpleAnnotationParserByTask) -> bool:
+    def execute_task(self, task_parser: SimpleAnnotationParserByTask, task_index: Optional[int] = None) -> bool:
         """
         1個のタスクに対してアノテーションを登録する。
 
@@ -395,7 +420,8 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
         if not self.confirm_processing(f"task_id={task_id} のアノテーションをインポートしますか？"):
             return False
 
-        logger.info(f"task_id={task_id} に対して処理します。")
+        logger_prefix = f"{str(task_index+1)} 件目: " if task_index is not None else ""
+        logger.info(f"{logger_prefix}task_id={task_id} に対して処理します。")
 
         task = self.service.wrapper.get_task_or_none(self.project_id, task_id)
         if task is None:
@@ -411,11 +437,14 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
         if self.is_force:
             if not can_put_annotation(task, self.service.api.account_id):
                 logger.debug(f"タスク'{task_id}' の担当者を自分自身に変更します。")
-                self.service.wrapper.change_task_operator(
-                    self.project_id, task_id, operator_account_id=self.service.api.account_id
+                old_account_id = task["account_id"]
+                task = self.service.wrapper.change_task_operator(
+                    self.project_id,
+                    task_id,
+                    operator_account_id=self.service.api.account_id,
+                    last_updated_datetime=task["updated_datetime"],
                 )
                 changed_operator = True
-                old_account_id = task["account_id"]
 
         else:
             if not can_put_annotation(task, self.service.api.account_id):
@@ -426,35 +455,76 @@ class ImportAnnotationMain(AbstractCommandLineWithConfirmInterface):
                 return False
 
         result_count = self.put_annotation_for_task(task_parser)
+        logger.info(f"{logger_prefix}タスク'{task_parser.task_id}'の入力データ {result_count} 個に対してアノテーションをインポートしました。")
+
         if changed_operator:
             logger.debug(f"タスク'{task_id}' の担当者を元に戻します。")
-            old_account_id = task["account_id"]
-            self.service.wrapper.change_task_operator(self.project_id, task_id, operator_account_id=old_account_id)
+            self.service.wrapper.change_task_operator(
+                self.project_id,
+                task_id,
+                operator_account_id=old_account_id,
+                last_updated_datetime=task["updated_datetime"],
+            )
 
         return result_count > 0
 
+    def execute_task_wrapper(
+        self,
+        tpl: tuple[int, SimpleAnnotationParserByTask],
+    ) -> bool:
+        task_index, task_parser = tpl
+        try:
+            return self.execute_task(task_parser, task_index=task_index)
+        except Exception:  # pylint: disable=broad-except
+            logger.warning(f"task_id={task_parser.task_id} のアノテーションのインポートに失敗しました。", exc_info=True)
+            return False
+
     def main(
         self,
         iter_task_parser: Iterator[SimpleAnnotationParserByTask],
-        target_task_ids: Optional[Set[str]] = None,
+        target_task_ids: Optional[set[str]] = None,
+        parallelism: Optional[int] = None,
     ):
+        def get_iter_task_parser_from_task_ids(
+            _iter_task_parser: Iterator[SimpleAnnotationParserByTask], _target_task_ids: set[str]
+        ) -> Iterator[SimpleAnnotationParserByTask]:
+            for task_parser in _iter_task_parser:
+                if task_parser.task_id in _target_task_ids:
+                    _target_task_ids.remove(task_parser.task_id)
+                    yield task_parser
+
+        if target_task_ids is not None:
+            # コマンドライン引数で --task_idが指定された場合は、対象のタスクのみインポートする
+            # tmp_target_task_idsが関数内で変更されるので、事前にコピーする
+            tmp_target_task_ids = copy.deepcopy(target_task_ids)
+            iter_task_parser = get_iter_task_parser_from_task_ids(iter_task_parser, tmp_target_task_ids)
+
         success_count = 0
-        for task_parser in iter_task_parser:
-            try:
-                if target_task_ids is not None and len(target_task_ids) > 0:
-                    # コマンドライン引数で --task_idが指定された場合は、対象のタスクのみインポートする
-                    if task_parser.task_id in target_task_ids:
-                        if self.execute_task(task_parser):
-                            success_count += 1
-                else:
-                    # コマンドライン引数で --task_idが指定されていない場合はすべてをインポートする
-                    if self.execute_task(task_parser):
+        task_count = 0
+        if parallelism is not None:
+            with multiprocessing.Pool(parallelism) as pool:
+                result_bool_list = pool.map(self.execute_task_wrapper, enumerate(iter_task_parser))
+                success_count = len([e for e in result_bool_list if e])
+                task_count = len(result_bool_list)
+
+        else:
+            for task_index, task_parser in enumerate(iter_task_parser):
+                try:
+                    result = self.execute_task(task_parser, task_index=task_index)
+                    if result:
                         success_count += 1
+                except Exception:
+                    logger.warning(f"task_id={task_parser.task_id} のアノテーションのインポートに失敗しました。", exc_info=True)
+                    continue
+                finally:
+                    task_count += 1
 
-            except Exception:  # pylint: disable=broad-except
-                logger.warning(f"task_id={task_parser.task_id} のアノテーションインポートに失敗しました。", exc_info=True)
+        if target_task_ids is not None and len(tmp_target_task_ids) > 0:
+            logger.warning(
+                f"'--task_id'で指定したタスクの内 {len(tmp_target_task_ids)} 件は、インポート対象のアノテーションデータに含まれていません。 :: {tmp_target_task_ids}"  # noqa: E501
+            )
 
-        logger.info(f"{success_count} 個のタスクに対してアノテーションをインポートしました。")
+        logger.info(f"{success_count} / {task_count} 件のタスクに対してアノテーションをインポートしました。")
 
 
 class ImportAnnotation(AbstractCommandLineInterface):
@@ -477,10 +547,17 @@ class ImportAnnotation(AbstractCommandLineInterface):
             )
             return False
 
-        elif annotation_path.is_file() and not zipfile.is_zipfile(str(annotation_path)):
+        elif not (zipfile.is_zipfile(str(annotation_path)) or annotation_path.is_dir()):
             print(f"{COMMON_MESSAGE} argument --annotation: ZIPファイルまたはディレクトリを指定してください。", file=sys.stderr)
             return False
 
+        if args.parallelism is not None and not args.yes:
+            print(
+                f"{COMMON_MESSAGE} argument --parallelism: '--parallelism'を指定するときは、必ず '--yes' を指定してください。",
+                file=sys.stderr,
+            )
+            return False
+
         return True
 
     def main(self):
@@ -489,7 +566,7 @@ class ImportAnnotation(AbstractCommandLineInterface):
             sys.exit(COMMAND_LINE_ERROR_STATUS_CODE)
 
         project_id = args.project_id
-        annotation_path = Path(args.annotation)
+        annotation_path: Path = args.annotation
 
         super().validate_project(project_id, [ProjectMemberRole.OWNER])
 
@@ -515,7 +592,7 @@ class ImportAnnotation(AbstractCommandLineInterface):
             is_force=args.force,
         )
 
-        main_obj.main(iter_task_parser, target_task_ids=target_task_ids)
+        main_obj.main(iter_task_parser, target_task_ids=target_task_ids, parallelism=args.parallelism)
 
 
 def main(args):
@@ -531,7 +608,7 @@ def parse_args(parser: argparse.ArgumentParser):
 
     parser.add_argument(
         "--annotation",
-        type=str,
+        type=Path,
         required=True,
         help="Simpleアノテーションと同じフォルダ構成のzipファイル or ディレクトリのパスを指定してください。" "タスクの状態が作業中/完了の場合はインポートしません。",
     )
@@ -558,6 +635,12 @@ def parse_args(parser: argparse.ArgumentParser):
         "--force", action="store_true", help="過去に割り当てられていて現在の担当者が自分自身でない場合、タスクの担当者を自分自身に変更してからアノテーションをインポートします。"
     )
 
+    parser.add_argument(
+        "--parallelism",
+        type=int,
+        help="並列度。指定しない場合は、逐次的に処理します。指定した場合は、``--yes`` も指定してください。",
+    )
+
     parser.set_defaults(subcommand_func=main)
 
 
