@@ -101,14 +101,14 @@ def add_parser(
         group.add_argument("--yes", action="store_true", help="処理中に現れる問い合わせに対して、常に ``yes`` と回答します。")
 
         group.add_argument(
-            "--endpoint_url", type=str, help=f"Annofab WebAPIのエンドポイントを指定します。指定しない場合は ``{DEFAULT_ENDPOINT_URL}`` です。"
+            "--endpoint_url", type=str, help=f"Annofab WebAPIのエンドポイントを指定します。", default=DEFAULT_ENDPOINT_URL
         )
 
         group.add_argument(
             "--logdir",
             type=Path,
             default=".log",
-            help="ログファイルを保存するディレクトリを指定します。指定しない場合は ``.log`` ディレクトリ'にログファイルが保存されます。",
+            help="ログファイルを保存するディレクトリを指定します。",
         )
 
         group.add_argument("--disable_log", action="store_true", help="ログを無効にします。")
@@ -359,7 +359,7 @@ def prompt_yesnoall(msg: str) -> Tuple[bool, bool]:
         msg: 確認メッセージ
 
     Returns:
-        Tuple[yesno, allflag]. yesno:Trueならyes. allflag: Trueならall.
+        Tuple[yesno, all_flag]. yesno:Trueならyes. all_flag: Trueならall.
 
     """
     while True:
@@ -414,7 +414,7 @@ class ArgumentParser:
         '--format` 引数を追加
         """
         if help_message is None:
-            help_message = f"出力フォーマットを指定します。指定しない場合は、{default.value} フォーマットになります。"
+            help_message = f"出力フォーマットを指定します。"
 
         self.parser.add_argument(
             "-f", "--format", type=str, choices=[e.value for e in choices], default=default.value, help=help_message
@@ -426,9 +426,9 @@ class ArgumentParser:
         """
         if help_message is None:
             help_message = (
-                "CSVのフォーマットをJSON形式で指定します。 ``--format`` が ``csv`` でないときは、このオプションは無視されます。"
-                " ``file://`` を先頭に付けると、JSON形式のファイルを指定できます。"
-                "指定した値は、`pandas.DataFrame.to_csv <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_csv.html>`_ の引数として渡されます。"  # noqa: E501
+                "CSVのフォーマットをJSON形式で指定します。 ``--format`` が ``csv`` でないときは、このオプションは無視されます。\n"
+                "``file://`` を先頭に付けると、JSON形式のファイルを指定できます。\n"
+                "指定した値は ``pandas.DataFrame.to_csv`` の引数として渡されます。"
             )
 
         self.parser.add_argument("--csv_format", type=str, help=help_message)
@@ -438,7 +438,7 @@ class ArgumentParser:
         '--output` 引数を追加
         """
         if help_message is None:
-            help_message = "出力先のファイルパスを指定します。指定しない場合は、標準出力に出力されます。"
+            help_message = "出力先のファイルパスを指定します。未指定の場合は、標準出力に出力されます。"
 
         self.parser.add_argument("-o", "--output", type=str, required=required, help=help_message)
 
@@ -455,9 +455,15 @@ class ArgumentParser:
         if help_message is None:
             help_message = (
                 "タスクを絞り込むためのクエリ条件をJSON形式で指定します。"
-                " ``file://`` を先頭に付けると、JSON形式のファイルを指定できます。"
-                "使用できるキーは、``task_id`` , ``phase`` , ``phase_stage`` , ``status`` , ``user_id`` ,"
-                " ``account_id`` , ``no_user``  (bool値)  のみです。"
+                " ``file://`` を先頭に付けると、JSON形式のファイルを指定できます。\n"
+                "以下のキーを指定できます。\n\n"
+                " * ``task_id``\n"
+                " * ``phase``\n"
+                " * ``phase_stage``\n"
+                " * ``status``\n"
+                " * ``user_id``\n"
+                " * ``account_id``\n"
+                " * ``no_user``"
             )
         self.parser.add_argument("-tq", "--task_query", type=str, required=required, help=help_message)
 
