@@ -32,7 +32,6 @@ logger = logging.getLogger(__name__)
 
 def _replace_df_task(task: Task, replacement_dict_by_user_id: Dict[str, str]) -> Task:
     df_output = task.df.copy()
-    replace_by_columns(df_output, replacement_dict_by_user_id, main_column="user_id", sub_columns=["username"])
 
     replace_by_columns(
         df_output,
@@ -106,10 +105,12 @@ def mask_visualization_dir(
     not_masked_biography_set: Optional[Set[str]] = None,
     not_masked_user_id_set: Optional[Set[str]] = None,
     minimal_output: bool = False,
-    exclude_masked_user_for_linegraph: bool = False,
-):
-    # TODO: validation
+    exclude_masked_user_for_line_graph: bool = False,
+) -> None:
     user_performance = project_dir.read_user_performance()
+    if user_performance.empty():
+        logger.warning(f"メンバごとの生産性と品質情報が空であるため、ユーザー情報をマスクできません。終了します。")
+        return
 
     # マスクするユーザの情報を取得する
     replacement_dict_by_user_id = create_replacement_dict_by_user_id(
@@ -132,7 +133,7 @@ def mask_visualization_dir(
     output_project_dir.write_user_performance_scatter_plot(masked_user_performance)
 
     user_id_list: Optional[List[str]] = None
-    if exclude_masked_user_for_linegraph:
+    if exclude_masked_user_for_line_graph:
         user_id_list = list(not_masked_user_id_set)
 
     # TODO: validation
@@ -142,11 +143,8 @@ def mask_visualization_dir(
 
     write_line_graph(masked_task, output_project_dir, user_id_list=user_id_list, minimal_output=minimal_output)
 
-    try:
-        worktime_per_date_user = project_dir.read_worktime_per_date_user()
-    except Exception:
-        logger.warning(f"'{project_dir}'のユーザごと日ごとの作業時間の読み込みに失敗しました。")
-    else:
+    worktime_per_date_user = project_dir.read_worktime_per_date_user()
+    if not worktime_per_date_user.is_empty():
         df_masked_worktime = create_masked_user_info_df(
             worktime_per_date_user.df,
             not_masked_biography_set=not_masked_biography_set,
@@ -155,40 +153,15 @@ def mask_visualization_dir(
         masked_worktime_per_date_user = WorktimePerDate(df_masked_worktime)
         output_project_dir.write_worktime_per_date_user(masked_worktime_per_date_user)
         output_project_dir.write_worktime_line_graph(masked_worktime_per_date_user, user_id_list=user_id_list)
+    else:
+        logger.warning(
+            f"'{project_dir.project_dir / project_dir.FILENAME_WORKTIME_PER_DATE_USER}'が存在しないかデータがないため、"
+            f"'{project_dir.FILENAME_WORKTIME_PER_DATE_USER}'から生成できるファイルを出力しません。"
+        )
 
     logger.debug(f"'{project_dir}'のマスクした結果を'{output_project_dir}'に出力しました。")
 
 
-def mask_visualization_root_dir(
-    project_root_dir: Path,
-    output_dir: Path,
-    not_masked_biography_set: Optional[Set[str]] = None,
-    not_masked_user_id_set: Optional[Set[str]] = None,
-    minimal_output: bool = False,
-    exclude_masked_user_for_linegraph: bool = False,
-):
-
-    for p_project_dir in project_root_dir.iterdir():
-        if not p_project_dir.is_dir():
-            continue
-
-        project_output_dir = ProjectDir(output_dir / p_project_dir.name)
-
-        try:
-            project_dir = ProjectDir(p_project_dir)
-            mask_visualization_dir(
-                project_dir,
-                project_output_dir,
-                not_masked_biography_set=not_masked_biography_set,
-                not_masked_user_id_set=not_masked_user_id_set,
-                minimal_output=minimal_output,
-                exclude_masked_user_for_linegraph=exclude_masked_user_for_linegraph,
-            )
-        except Exception:  # pylint: disable=broad-except
-            logger.warning(f"'{project_dir}'のユーザのマスク処理に失敗しました。", exc_info=True)
-            continue
-
-
 def main(args):
     not_masked_biography_set = (
         set(get_list_from_args(args.not_masked_biography)) if args.not_masked_biography is not None else None
@@ -197,13 +170,13 @@ def main(args):
         set(get_list_from_args(args.not_masked_user_id)) if args.not_masked_user_id is not None else None
     )
 
-    mask_visualization_root_dir(
-        project_root_dir=args.dir,
+    mask_visualization_dir(
+        project_dir=ProjectDir(args.dir),
+        output_project_dir=ProjectDir(args.output_dir),
         not_masked_biography_set=not_masked_biography_set,
         not_masked_user_id_set=not_masked_user_id_set,
-        output_dir=args.output_dir,
         minimal_output=args.minimal,
-        exclude_masked_user_for_linegraph=args.exclude_masked_user_for_linegraph,
+        exclude_masked_user_for_line_graph=args.exclude_masked_user_for_line_graph,
     )
 
 
@@ -213,7 +186,7 @@ def parse_args(parser: argparse.ArgumentParser):
         "--dir",
         type=Path,
         required=True,
-        help=f"マスクしたいプロジェクトディレクトリが存在するディレクトリを指定してください。プロジェクトディレクトリは  ``annofabcli statistics visualize`` コマンドの出力結果です。",  # noqa: E501
+        help=f"マスクしたいプロジェクトディレクトリを指定してください。プロジェクトディレクトリは  ``annofabcli statistics visualize`` コマンドの出力結果です。",  # noqa: E501
     )
 
     parser.add_argument(
@@ -237,12 +210,12 @@ def parse_args(parser: argparse.ArgumentParser):
     )
 
     parser.add_argument(
-        "--exclude_masked_user_for_linegraph",
+        "--exclude_masked_user_for_line_graph",
         action="store_true",
         help="折れ線グラフに、マスクされたユーザをプロットしません。",
     )
 
-    parser.add_argument("-o", "--output_dir", type=Path, required=True, help="出力先ディレクトリ。配下にプロジェクトディレクトリが生成されます。")
+    parser.add_argument("-o", "--output_dir", type=Path, required=True, help="出力先ディレクトリ。")
 
     parser.set_defaults(subcommand_func=main)
 
